@@ -292,14 +292,8 @@ def rebuild_collect_shared(
                 f" shared_var.type={store_into.type},"
                 f" update_val={update_val}, update_val.type={getattr(update_val, 'type', None)})."
             )
-            err_sug = (
-                "If the difference is related to the broadcast pattern,"
-                " you can call the"
-                " tensor.shape.unbroadcast(var, axis_to_unbroadcast[, ...])"
-                " function to mask broadcastable dimensions."
-            )
 
-            raise TypeError(err_msg, err_sug)
+            raise TypeError(err_msg)
         assert store_into.type.is_super(update_val.type)
 
         update_d[store_into] = update_val
@@ -377,6 +371,7 @@ def pfunc(
     on_unused_input=None,
     output_keys=None,
     fgraph: FunctionGraph | None = None,
+    trust_input: bool = False,
 ) -> Function:
     """
     Function-constructor for graphs with shared variables.
@@ -425,6 +420,12 @@ def pfunc(
     fgraph
         An existing `FunctionGraph` from which to construct the returned
         `Function`.  When this is non-``None``, nothing is cloned.
+    trust_input : bool, default False
+        If True, no input validation checks are performed when the function is
+        called. This includes checking the number of inputs, their types and
+        that multiple inputs are not aliased to each other. Failure to meet any
+        of these conditions can lead to computational errors or to the
+        interpreter crashing.
 
     Returns
     -------
@@ -472,6 +473,7 @@ def pfunc(
         on_unused_input=on_unused_input,
         output_keys=output_keys,
         fgraph=fgraph,
+        trust_input=trust_input,
     )
 
 
@@ -569,7 +571,7 @@ def construct_pfunc_ins_and_outs(
     if not fgraph:
         # Extend the outputs with the updates on input variables so they are
         # also cloned
-        additional_outputs = [i.update for i in inputs if i.update]
+        additional_outputs = [i.update for i in inputs if i.update is not None]
         if outputs is None:
             out_list = []
         else:
@@ -603,12 +605,12 @@ def construct_pfunc_ins_and_outs(
 
         new_inputs = []
 
-        for i, iv in zip(inputs, input_variables):
+        for i, iv in zip(inputs, input_variables, strict=True):
             new_i = copy(i)
             new_i.variable = iv
 
             # If needed, replace the input's update by its cloned equivalent
-            if i.update:
+            if i.update is not None:
                 new_i.update = clone_d[i.update]
 
             new_inputs.append(new_i)
@@ -637,13 +639,13 @@ def construct_pfunc_ins_and_outs(
         assert len(fgraph.inputs) == len(inputs)
         assert len(fgraph.outputs) == len(outputs)
 
-        for fg_inp, inp in zip(fgraph.inputs, inputs):
+        for fg_inp, inp in zip(fgraph.inputs, inputs, strict=True):
             if fg_inp != getattr(inp, "variable", inp):
                 raise ValueError(
                     f"`fgraph`'s input does not match the provided input: {fg_inp}, {inp}"
                 )
 
-        for fg_out, out in zip(fgraph.outputs, outputs):
+        for fg_out, out in zip(fgraph.outputs, outputs, strict=True):
             if fg_out != getattr(out, "variable", out):
                 raise ValueError(
                     f"`fgraph`'s output does not match the provided output: {fg_out}, {out}"
