@@ -2,7 +2,6 @@ import inspect
 import sys
 import time
 import warnings
-from collections import OrderedDict
 from functools import partial
 from io import StringIO
 
@@ -324,7 +323,7 @@ class Feature:
         might be broken for all intents and purposes.
 
         """
-        return OrderedDict()
+        return {}
 
     def clone(self):
         """Create a clone that can be attached to a new `FunctionGraph`.
@@ -492,7 +491,7 @@ class Validator(Feature):
                 if verbose:
                     r = uf.f_locals.get("r", "")
                     reason = uf_info.function
-                    print(f"validate failed on node {r}.\n Reason: {reason}, {e}")
+                    print(f"validate failed on node {r}.\n Reason: {reason}, {e}")  # noqa: T201
                 raise
         t1 = time.perf_counter()
         if fgraph.profile:
@@ -568,6 +567,13 @@ class ReplaceValidate(History, Validator):
         if verbose is None:
             verbose = config.optimizer_verbose
 
+        if verbose:
+            print_reason = True
+            if config.optimizer_verbose_ignore:
+                print_reason = str(reason) not in config.optimizer_verbose_ignore.split(
+                    ","
+                )
+
         for r, new_r in replacements:
             try:
                 fgraph.replace(r, new_r, reason=reason, verbose=False, **kwargs)
@@ -604,13 +610,13 @@ class ReplaceValidate(History, Validator):
         except Exception as e:
             fgraph.revert(chk)
             if verbose:
-                print(
+                print(  # noqa: T201
                     f"rewriting: validate failed on node {r}.\n Reason: {reason}, {e}"
                 )
             raise
 
-        if verbose:
-            print(
+        if verbose and print_reason:
+            print(  # noqa: T201
                 f"rewriting: rewrite {reason} replaces {r} of {r.owner} with {new_r} of {new_r.owner}"
             )
 
@@ -693,11 +699,11 @@ class NodeFinder(Bookkeeper):
         except TypeError:  # node.op is unhashable
             return
         except Exception as e:
-            print("OFFENDING node", type(node), type(node.op), file=sys.stderr)
+            print("OFFENDING node", type(node), type(node.op), file=sys.stderr)  # noqa: T201
             try:
-                print("OFFENDING node hash", hash(node.op), file=sys.stderr)
+                print("OFFENDING node hash", hash(node.op), file=sys.stderr)  # noqa: T201
             except Exception:
-                print("OFFENDING node not hashable", file=sys.stderr)
+                print("OFFENDING node not hashable", file=sys.stderr)  # noqa: T201
             raise e
 
     def on_prune(self, fgraph, node, reason):
@@ -726,7 +732,7 @@ class PrintListener(Feature):
 
     def on_attach(self, fgraph):
         if self.active:
-            print("-- attaching to: ", fgraph)
+            print("-- attaching to: ", fgraph)  # noqa: T201
 
     def on_detach(self, fgraph):
         """
@@ -734,19 +740,19 @@ class PrintListener(Feature):
         that it installed into the function_graph
         """
         if self.active:
-            print("-- detaching from: ", fgraph)
+            print("-- detaching from: ", fgraph)  # noqa: T201
 
     def on_import(self, fgraph, node, reason):
         if self.active:
-            print(f"-- importing: {node}, reason: {reason}")
+            print(f"-- importing: {node}, reason: {reason}")  # noqa: T201
 
     def on_prune(self, fgraph, node, reason):
         if self.active:
-            print(f"-- pruning: {node}, reason: {reason}")
+            print(f"-- pruning: {node}, reason: {reason}")  # noqa: T201
 
     def on_change_input(self, fgraph, node, i, r, new_r, reason=None):
         if self.active:
-            print(f"-- changing ({node}.inputs[{i}]) from {r} to {new_r}")
+            print(f"-- changing ({node}.inputs[{i}]) from {r} to {new_r}")  # noqa: T201
 
 
 class PreserveVariableAttributes(Feature):
